/************************************************/
/*												*/
/*	Code on BF-532 for "PDAP-BF Interface		*/
/*												*/
/*												*/
/*		- Initial Version						*/
/*				7th August, 2004				*/
/*												*/
/*						kunal singh		 		*/
/*						DSP Apps Engineer		*/
/*						Analog Devices			*/
/*												*/
/************************************************/		

#include <stdio.h>
#include <cdefBF532.h>
#include <signal.h>
#include <sys/exception.h>
	

#define BUFFER_SIZE 256
/****************************************/
/*										*/
/*	Size of the Data Buffer				*/
/*										*/
/* 	This is also the maximum limit on 	*/
/*	the DMA count for a single transfer	*/
/*										*/
/****************************************/


volatile short *AsyncMem ;


EX_INTERRUPT_HANDLER(request);
EX_INTERRUPT_HANDLER(acknowledge);
/************************************************/
/*												*/
/*	Interrupt Handlers							*/
/*												*/
/*	request() : Edge sensitive interrupt on 	*/
/*				PF5, mapped to IVG11			*/
/*												*/
/*												*/
/*	acknowledge() : Edge sensitive interrupt	*/ 
/*					on	PF5, mapped to IVG12	*/
/*												*/
/************************************************/



void InitAsyncMemory();
void Init_Data();
void initialize_PF_intrpts();
void generate_interrupt();
void drive_data ();
/****************************/
/*							*/
/*	Function Declarations 	*/												
/*							*/
/****************************/




unsigned int dataSrc[BUFFER_SIZE] ;
/************************************/
/*									*/
/*	Data Buffer for DMA transfers	*/
/*									*/
/************************************/



unsigned int command_flag;
unsigned int request_flag;
/************************************/
/*									*/
/*	Flags to keep track of command	*/
/*	and Data phase					*/
/*									*/
/************************************/


unsigned int dma_count;
unsigned int dma_index;
/****************************************/
/*										*/
/*	Variables to hold DMA parameters	*/												
/*										*/
/****************************************/


unsigned int temp;
/************************************/
/*									*/
/*	Variable for Temporary Storage	*/
/*									*/
/************************************/




/********************************/
/*								*/
/*		Main Function 			*/
/*								*/
/********************************/
main ()
{
	InitAsyncMemory () ;
	// Initialize the EBIU
	
	Init_Data();
	// Intialize the Data Buffer
	// with the data values to be transfered
	
	initialize_PF_intrpts();
	// Configure the PF4 as o/p
	// and PF,PF6 as positive edge triggered interrupts
	
	register_handler(ik_ivg11, request);
	register_handler(ik_ivg12, acknowledge);
	// Define interrupt handler for IVG11, IVG12
	// IVG11: mapped to PF5
	// IVG12: mapped to PF6
	
	while (1)
	{
		
	}		
}



/****************************************/
/*										*/
/*	Initialize the EBIU Registers for 	*/
/*	Aynchronous Memory Bank				*/
/*										*/
/*	The Asyn Mem Signal are connected	*/
/*	to the PDAP							*/
/*										*/
/****************************************/
void InitAsyncMemory(void)
{
	(*pEBIU_AMGCTL) = 0x3 ;
	// Enable clock out for Async accesses
	// Enable only Bank 0
	
	
	(*pEBIU_AMBCTL0) = 0x00001110 ;	
	// Set up bank 0 parameters -- 
	// Setup time = 1 cycle
	// Write access = 1 cycle
	// Hold = 0 
}


void Init_Data()
{

	int i;
	

	for (i=0; i<BUFFER_SIZE; ++i)
	{
		dataSrc[i] = 0xcafebead + i;	
	}
	// Initialize the Data Buffer

}
	


void initialize_PF_intrpts()
{
	
	// configure PF4 as output
	
	// configure PF5 and PF6 as inputs
	
	// PF5 and PF6 are rising edge sensitive interrupts
	
	// PF5 is mapped to intr_A/IVG11 and PF6 is mapped to intr_B/IVG12
	
	temp = *pFIO_DIR;
	temp &= 0xFF9F;	
	temp |= 0x0010;	
	*pFIO_DIR = temp;
	// Define PF4 as output
	// Define PF5, PF6 as inputs
	
	*pFIO_INEN = 0x0060;
	// enable the i/p buffer for PF5, PF6
	
	*pFIO_MASKA_D = 0x0020;
	*pFIO_MASKB_D = 0x0060;
	// enable the PF5 for the Interrupt A
	// enable the PF6 for the Interrupt B
	
	
	*pFIO_POLAR = 0;
	// set the active-high/rising-edge polarity
	
	*pFIO_EDGE = 0x0060;
	*pFIO_BOTH = 0;
	// edge sensitive interrupt for PF5, PF6
	// latch interrupt only on a single edge
	
	
	temp = *pSIC_IMASK;
	temp |= 0x00180000;
	*pSIC_IMASK = temp;
	// enable the PF intr A and B
	
	temp = *pSIC_IAR2;
	temp &= 0xFFF00FFF;
	temp |= 0x00054000;
	*pSIC_IAR2 = temp;
	// map the PF intr A to ivg11, and PF intr B to ivg12

	
}





EX_INTERRUPT_HANDLER(request)
{
	
	printf("\n enter the address location in the ADSP-21365\n");
	scanf("%u",&dma_index);

	printf("\n enter the number of transfers \n");
	scanf("%u",&dma_count);

	request_flag = 1;
	command_flag = 1;
	
	generate_interrupt();
	// Generate Interupt Request for ADSP-21365
	// By toggling the PF4
}


void generate_interrupt()
{
	int wait;
	
	
	*pFIO_FLAG_C = 0x0010;
	// Clear the PF4
	
	for (wait = 0; wait<40; wait++)
	{
	}
	
	
	*pFIO_FLAG_S = 0x0010;
	// Set the PF4
	// Causes a low-high-low transition on PF4
	

}




EX_INTERRUPT_HANDLER(acknowledge)
{
		
	if (request_flag == 1)
	// Valid Acknowledge
	{
		
		if(command_flag == 1)
		{
			// send command word to the ADSP-21365
	
			(*AsyncMem) = dma_index & 0xffff ;
			(*AsyncMem) = (dma_index >> 16) & 0xffff ;
			// send dma_index
				
			(*AsyncMem) = dma_count & 0xffff ;
			(*AsyncMem) = (dma_count >> 16) & 0xffff ;
			// send dma_count

			command_flag = 0;
		}
		else
		{
			

			drive_data ();
			// send data to the ADSP-21365
			request_flag = 0;
			
		}
	}
	
	else
	// Invalid Acknowlege Interrupt
	{
		printf("\n An error has occured while transmitting\n");
		request_flag = 0;
		command_flag = 0;
	
	}

}


void drive_data ()
{
	int i = 0 ;
	
	// Point to the external async memory bank

	AsyncMem = (volatile short *)0x20000000 ;
	
	for (i = 0; i < dma_count; ++i)
	{
		(*AsyncMem) = dataSrc[i] & 0xffff ;
		(*AsyncMem) = (dataSrc[i] >> 16) & 0xffff ;
		
	}
}


